@@ -59,7 +59,7 @@ public:
 	void processFullPeer(UserData *user, const MTPUserFull &result);
 
 	void requestSelfParticipant(ChannelData *channel);
-	void kickParticipant(PeerData *peer, UserData *user);
+	void kickParticipant(PeerData *peer, UserData *user, const MTPChannelBannedRights &currentRights);
 	void unblockParticipant(PeerData *peer, UserData *user);
 
 	void requestWebPageDelayed(WebPageData *page);
@@ -91,6 +91,9 @@ public:
 
 	bool isQuitPrevent();
 
+	void applyUpdatesNoPtsCheck(const MTPUpdates &updates);
+	void applyUpdateNoPtsCheck(const MTPUpdate &update);
+
 	~ApiWrap();
 
 private:
