@@ -323,9 +323,6 @@ TabbedSelector::TabbedSelector(QWidget *parent, gsl::not_null<Window::Controller
 				_scroll->disableScroll(disabled);
 			}
 		});
-		connect(widget, &Inner::saveConfigDelayed, this, [this](int delay) {
-			AuthSession::Current().saveDataDelayed(delay);
-		});
 	}
 
 	connect(stickers(), SIGNAL(scrollUpdated()), this, SLOT(onScroll()));
@@ -653,7 +650,7 @@ void TabbedSelector::switchTab() {
 	auto slidingRect = QRect(_tabsSlider->x() * cIntRetinaFactor(), _scroll->y() * cIntRetinaFactor(), _tabsSlider->width() * cIntRetinaFactor(), (height() - _scroll->y()) * cIntRetinaFactor());
 	_slideAnimation->setFinalImages(direction, std::move(wasCache), std::move(nowCache), slidingRect, wasSectionIcons);
 	auto corners = App::cornersMask(ImageRoundRadius::Small);
-	_slideAnimation->setCornerMasks(QImage(*corners[0]), QImage(*corners[1]), QImage(*corners[2]), QImage(*corners[3]));
+	_slideAnimation->setCornerMasks(corners[0], corners[1], corners[2], corners[3]);
 	_slideAnimation->start();
 
 	hideForSliding();
