@@ -20,7 +20,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 */
 #include "observer_peer.h"
 
-#include "core/observer.h"
+#include "base/observer.h"
 
 namespace App {
 // Temp forward declaration (while all peer updates are not done through observers).
