@@ -34,43 +34,41 @@ struct PeerUpdate {
 	PeerData *peer;
 
 	enum class Flag {
+		None                      = 0x00000000U,
+
 		// Common flags
-		NameChanged            = 0x00000001U,
-		UsernameChanged        = 0x00000002U,
-		PhotoChanged           = 0x00000004U,
-		AboutChanged           = 0x00000008U,
-		NotificationsEnabled   = 0x00000010U,
-		SharedMediaChanged     = 0x00000020U,
-		MigrationChanged       = 0x00000040U,
-		PinnedChanged          = 0x00000080U,
+		NameChanged               = 0x00000001U,
+		UsernameChanged           = 0x00000002U,
+		PhotoChanged              = 0x00000004U,
+		AboutChanged              = 0x00000008U,
+		NotificationsEnabled      = 0x00000010U,
+		SharedMediaChanged        = 0x00000020U,
+		MigrationChanged          = 0x00000040U,
+		PinnedChanged             = 0x00000080U,
 
 		// For chats and channels
-		InviteLinkChanged      = 0x00000100U,
-		MembersChanged         = 0x00000200U,
-		AdminsChanged          = 0x00000400U,
-		BlockedUsersChanged    = 0x00000800U,
+		InviteLinkChanged         = 0x00000100U,
+		MembersChanged            = 0x00000200U,
+		AdminsChanged             = 0x00000400U,
+		BannedUsersChanged       = 0x00000800U,
 
 		// For users
-		UserCanShareContact    = 0x00010000U,
-		UserIsContact          = 0x00020000U,
-		UserPhoneChanged       = 0x00040000U,
-		UserIsBlocked          = 0x00080000U,
-		BotCommandsChanged     = 0x00100000U,
-		UserOnlineChanged      = 0x00200000U,
-		BotCanAddToGroups      = 0x00400000U,
-		UserCommonChatsChanged = 0x00800000U,
-		UserHasCalls           = 0x01000000U,
+		UserCanShareContact       = 0x00010000U,
+		UserIsContact             = 0x00020000U,
+		UserPhoneChanged          = 0x00040000U,
+		UserIsBlocked             = 0x00080000U,
+		BotCommandsChanged        = 0x00100000U,
+		UserOnlineChanged         = 0x00200000U,
+		BotCanAddToGroups         = 0x00400000U,
+		UserCommonChatsChanged    = 0x00800000U,
+		UserHasCalls              = 0x01000000U,
 
 		// For chats
-		ChatCanEdit            = 0x00010000U,
+		ChatCanEdit               = 0x00010000U,
 
 		// For channels
-		ChannelAmIn            = 0x00010000U,
-		ChannelAmEditor        = 0x00020000U,
-		ChannelCanEditPhoto    = 0x00040000U,
-		ChannelCanAddMembers   = 0x00080000U,
-		ChannelCanViewAdmins   = 0x00100000U,
-		ChannelCanViewMembers  = 0x00200000U,
+		ChannelAmIn               = 0x00010000U,
+		ChannelRightsChanged      = 0x00020000U,
 	};
 	Q_DECLARE_FLAGS(Flags, Flag);
 	Flags flags = 0;
