@@ -18,7 +18,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "platform/linux/specific_linux.h"
 
 #include "platform/linux/linux_libs.h"
-#include "lang.h"
+#include "lang/lang_keys.h"
 #include "application.h"
 #include "mainwidget.h"
 #include "mainwindow.h"
@@ -278,16 +278,6 @@ void psActivateProcess(uint64 pid) {
 //	objc_activateProgram();
 }
 
-QString psCurrentCountry() {
-	QString country;// = objc_currentCountry();
-	return country.isEmpty() ? QString::fromLatin1(DefaultCountry) : country;
-}
-
-QString psCurrentLanguage() {
-	QString lng;// = objc_currentLang();
-	return lng.isEmpty() ? QString::fromLatin1(DefaultLanguage) : lng;
-}
-
 namespace {
 
 QString getHomeDir() {
@@ -402,6 +392,14 @@ bool TranslucentWindowsSupported(QPoint globalPosition) {
 	return false;
 }
 
+QString SystemCountry() {
+	return QString();
+}
+
+QString SystemLanguage() {
+	return QString();
+}
+
 namespace ThirdParty {
 
 void start() {
@@ -541,8 +539,21 @@ bool _execUpdater(bool update = true, const QString &crashreport = QString()) {
 	QByteArray data(QFile::encodeName(cExeDir() + (update ? "Updater" : gExeName)));
 	memcpy(path, data.constData(), data.size());
 
-	char *args[MaxArgsCount] = {0}, p_noupdate[] = "-noupdate", p_autostart[] = "-autostart", p_debug[] = "-debug", p_tosettings[] = "-tosettings", p_key[] = "-key", p_path[] = "-workpath", p_startintray[] = "-startintray", p_testmode[] = "-testmode", p_crashreport[] = "-crashreport";
-	char p_datafile[MaxLen] = {0}, p_pathbuf[MaxLen] = {0}, p_crashreportbuf[MaxLen] = {0};
+	char *args[MaxArgsCount] = { 0 };
+	char p_noupdate[] = "-noupdate";
+	char p_autostart[] = "-autostart";
+	char p_debug[] = "-debug";
+	char p_tosettings[] = "-tosettings";
+	char p_key[] = "-key";
+	char p_datafile[MaxLen] = { 0 };
+	char p_path[] = "-workpath";
+	char p_pathbuf[MaxLen] = { 0 };
+	char p_startintray[] = "-startintray";
+	char p_testmode[] = "-testmode";
+	char p_crashreport[] = "-crashreport";
+	char p_crashreportbuf[MaxLen] = { 0 };
+	char p_exe[] = "-exename";
+	char p_exebuf[MaxLen] = { 0 };
 	int argIndex = 0;
 	args[argIndex++] = path;
 	if (!update) {
@@ -575,6 +586,12 @@ bool _execUpdater(bool update = true, const QString &crashreport = QString()) {
 			args[argIndex++] = p_crashreportbuf;
 		}
 	}
+	QByteArray exef = QFile::encodeName(cExeName());
+	if (exef.size() > 0 && exef.size() < MaxLen) {
+		memcpy(p_exebuf, exef.constData(), exef.size());
+		args[argIndex++] = p_exe;
+		args[argIndex++] = p_exebuf;
+	}
 
 	Logs::closeMain();
 	SignalHandlers::finish();
