@@ -30,9 +30,13 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "apiwrap.h"
 #include "auth_session.h"
 #include "lang/lang_keys.h"
+#include "profile/profile_channel_controllers.h"
 
 namespace Profile {
 
+constexpr auto kEnableSearchMembersAfterCount = 50;
+constexpr auto kMaxChannelMembersDeleteAllowed = 1000;
+
 using UpdateFlag = Notify::PeerUpdate::Flag;
 
 ActionsWidget::ActionsWidget(QWidget *parent, PeerData *peer) : BlockWidget(parent, peer, lang(lng_profile_actions_section)) {
@@ -59,6 +63,12 @@ void ActionsWidget::notifyPeerUpdated(const Notify::PeerUpdate &update) {
 				return true;
 			}
 		}
+		if (update.flags & UpdateFlag::MembersChanged) {
+			if (peer()->isMegagroup()) {
+				// Search members button could change.
+				return true;
+			}
+		}
 		return false;
 	};
 	if (needFullRefresh()) {
@@ -142,6 +152,9 @@ void ActionsWidget::refreshButtons() {
 		addButton(lang(lng_profile_clear_history), SLOT(onClearHistory()));
 		addButton(lang(lng_profile_clear_and_exit), SLOT(onDeleteConversation()));
 	} else if (auto channel = peer()->asChannel()) {
+		if (channel->isMegagroup() && channel->membersCount() > kEnableSearchMembersAfterCount) {
+			addButton(lang(lng_profile_search_members), SLOT(onSearchMembers()));
+		}
 		if (!channel->amCreator() && (!channel->isMegagroup() || channel->isPublic())) {
 			addButton(lang(lng_profile_report), SLOT(onReport()));
 		}
@@ -324,6 +337,13 @@ void ActionsWidget::onUpgradeToSupergroup() {
 }
 
 void ActionsWidget::onDeleteChannel() {
+	if (auto channel = peer()->asChannel()) {
+		if (channel->membersCount() > kMaxChannelMembersDeleteAllowed) {
+			Ui::show(Box<InformBox>((channel->isMegagroup() ? lng_cant_delete_group : lng_cant_delete_channel)(lt_count, kMaxChannelMembersDeleteAllowed)));
+			return;
+		}
+	}
+
 	auto text = lang(peer()->isMegagroup() ? lng_sure_delete_group : lng_sure_delete_channel);
 	Ui::show(Box<ConfirmBox>(text, lang(lng_box_delete), st::attentionBoxButton, base::lambda_guarded(this, [this] {
 		Ui::hideLayer();
@@ -347,6 +367,12 @@ void ActionsWidget::onLeaveChannel() {
 	})));
 }
 
+void ActionsWidget::onSearchMembers() {
+	if (auto channel = peer()->asChannel()) {
+		ParticipantsBoxController::Start(channel, ParticipantsBoxController::Role::Members);
+	}
+}
+
 void ActionsWidget::onReport() {
 	Ui::show(Box<ReportBox>(peer()));
 }
