@@ -65,25 +65,39 @@ public:
 		return _gifPauseLevelChanged;
 	}
 	bool isGifPausedAtLeastFor(GifPauseReason reason) const;
+	base::Observable<void> &floatPlayerAreaUpdated() {
+		return _floatPlayerAreaUpdated;
+	}
 
 	struct ColumnLayout {
 		int bodyWidth;
 		int dialogsWidth;
+		int chatWidth;
 		Adaptive::WindowLayout windowLayout;
 	};
-	ColumnLayout computeColumnLayout();
+	ColumnLayout computeColumnLayout() const;
 	int dialogsSmallColumnWidth() const;
-	bool provideChatWidth(int requestedWidth);
+	bool canProvideChatWidth(int requestedWidth) const;
+	void provideChatWidth(int requestedWidth);
 
 	base::Variable<float64> &dialogsWidthRatio() {
 		return _dialogsWidthRatio;
 	}
+	const base::Variable<float64> &dialogsWidthRatio() const {
+		return _dialogsWidthRatio;
+	}
 	base::Variable<bool> &dialogsListFocused() {
 		return _dialogsListFocused;
 	}
+	const base::Variable<bool> &dialogsListFocused() const {
+		return _dialogsListFocused;
+	}
 	base::Variable<bool> &dialogsListDisplayForced() {
 		return _dialogsListDisplayForced;
 	}
+	const base::Variable<bool> &dialogsListDisplayForced() const {
+		return _dialogsListDisplayForced;
+	}
 
 private:
 	gsl::not_null<MainWindow*> _window;
@@ -93,6 +107,7 @@ private:
 
 	GifPauseReasons _gifPauseReasons = { 0 };
 	base::Observable<void> _gifPauseLevelChanged;
+	base::Observable<void> _floatPlayerAreaUpdated;
 
 	base::Variable<float64> _dialogsWidthRatio = { kDefaultDialogsWidthRatio };
 	base::Variable<bool> _dialogsListFocused = { false };
